#!/usr/bin/env python

listsdir = ""
cwdir = ""

def sorting(list1,list2):
	#Sorter function for list of lists
	x = list1[0]
	y = list2[0]
	if x>y: return 1
	elif x==y: return 0
	elif x<y: return -1

class ElectroCatalogue:
	def __init__(self, listsdirectory, cwdirectory):
		global listsdir
		global cwdir
		listsdir = listsdirectory #the directory of the database
		cwdir = cwdirectory
		self.load = []
		self.category = ""
	
	def reset_all(self):
	
		#This method erases the whole database as is! Use it with caution!
		#Opening each file for reading and closing, which erases their contents.
		
		file = open(listsdir+"/transistors","w")
		file.close()
		file = open(listsdir+"/resistors","w")
		file.close()
		file = open(listsdir+"/ics","w")
		file.close()
		file = open(listsdir+"/diodes","w")
		file.close()
		file = open(listsdir+"/buttons","w")
		file.close()
		file = open(listsdir+"/capacitors","w")
		file.close()
		file = open(listsdir+"/others","w")
		file.close()
	
	def load_category(self, category):
		
		#The following method loads in the contents of the given category.
		#The category file is opened. It's lines are being read and decoded by removing the "@"s from between the attributes stored in
		#the line. The list of attributes are then appended to a list forming a 2 dimensional list of lists.
		#The number of elements in the list is then returned.
		
		self.load = []
		self.category = category
		#opening file
		filein = open(listsdir+"/"+category+"s","r")
		#decoding the lines (by removing the "@"s from between the attributes)
		line = filein.readline()
		while line != "":
			type = line.partition("@")[0]
			quantity = int(line.partition("@")[2].partition("@")[0])
			place = line.partition("@")[2].partition("@")[2].partition("@")[0]
			description = line.partition("@")[2].partition("@")[2].partition("@")[2].strip("\n")
			self.load.append([type, quantity, place, description])
			line = filein.readline()
		#the number of loaded elements is returned
		return len(self.load)
	
	def set_attributes(self,category,type,quantity=-1,place="@",description="@",newtype="@"):
	
		#The following method searches for the element with the name "type" in the "category" category.
		#Firstly a temporary list is populated with the elements of "category", then the "type" is searched for in the list.
		#When "type" is found, it's attributes are changed with the given ones within the list.
		#Finally the list is written out to a file. The list is then freed.
		
		loading = [] #temporary list of elements
		#opening category file for reading
		filein = open(listsdir+"/"+category+"s","r")
		#populating the list with the elements from the category
		line = filein.readline()
		while line != "":
			typein = line.partition("@")[0]
			quantityin = int(line.partition("@")[2].partition("@")[0])
			placein = line.partition("@")[2].partition("@")[2].partition("@")[0]
			descriptionin = line.rpartition("@")[2].strip("\n")
			loading.append([typein, quantityin, placein, descriptionin])
			line = filein.readline()
		#closing file
		filein.close()
		
		#searching for the given name (type)
		#if found changing the element's attributes
		j = 0
		found = 0
		while j<len(loading):
			if loading[j][0] == type:
				if newtype != "@": loading[j][0] = newtype
				if quantity != -1: loading[j][1] = quantity
				if place != "@": loading[j][2] = place
				if description != "@": loading[j][3] = description
				found = 1
				break
			j = j+1
		#if type and newtype are not the same, resorting the list
		if type != newtype: loading.sort(sorting)
		#opening file to write out to
		fileout = open(listsdir+"/"+category+"s","w")
		#writing out
		j = 0
		while j<len(loading):
			fileout.write(loading[j][0]+"@"+str(loading[j][1])+"@"+loading[j][2]+"@"+loading[j][3]+"\n")
			j = j+1
		fileout.close()
		#freeing list (erasing it's contents)
		loading = []
		if found == 1: return 1
		else: return 0

	def get_attribute(self,category,type,attrib):
	
		#The following method returns an attribute of an element with the name "type".
		#Firstly the category's file is opened for reading and searched for "type".
		#When "type" is found, it's attributes are being loaded.
		#The asked attribute is then returned. If the "type" is not found, -1 is returned.
		
		type = type.upper() #uppercasing the search term, so the case doesen't matter
		filein = open(listsdir+"/"+category+"s","r") #opening file for reading
		line = filein.readline()
		while line != "":
			if line.partition("@")[0].upper() == type:
				break
			line = filein.readline()
		if line == "": return -1
		else:
			quantity = int(line.partition("@")[2].partition("@")[0])
			place = line.partition("@")[2].partition("@")[2].partition("@")[0]
			description = line.rpartition("@")[2].strip("\n")
			if attrib == "quantity": return quantity
			elif attrib == "place":  return place
			elif attrib == "description": return description
	
	def add_element(self,category,type,quantity,place,description):
		
		#This method adds an element to the given category then refreshes the category file.
		
		loading = []
		#opening file
		filein = open(listsdir+"/"+category+"s","r")
		#decoding the lines (by removing the "@"s from between the attributes)
		line = filein.readline()
		while line != "":
			typein = line.partition("@")[0]
			quantityin = int(line.partition("@")[2].partition("@")[0])
			placein = line.partition("@")[2].partition("@")[2].partition("@")[0]
			descriptionin = line.partition("@")[2].partition("@")[2].partition("@")[2].strip("\n")
			loading.append([typein, quantityin, placein, descriptionin])
			line = filein.readline()
		#adding new element
		loading.append([type, quantity, place, description])
		loading.sort(sorting)
		filein.close()
		
		#opening file for writing and writing out
		fileout = open(listsdir+"/"+category+"s","w")
		j = 0
		while j<len(loading):
			fileout.write(loading[j][0]+"@"+str(loading[j][1])+"@"+loading[j][2]+"@"+loading[j][3]+"\n")
			j = j+1
		fileout.close()
		loading = []
	
	def remove_element(self,category,type):
		
		#Removes the element with type "type".
		#Firstly the category's elements are loaded into a temporary list. A second list is then populated with the elements
		#which have to be kept. The second list is written out.
		
		loading = []
		for i in type:
			i = i.upper()
		#opening file
		filein = open(listsdir+"/"+category+"s","r")
		#decoding the lines (by removing the "@"s from between the attributes)
		line = filein.readline()
		while line != "":
			typein = line.partition("@")[0]
			quantityin = line.partition("@")[2].partition("@")[0]
			placein = line.partition("@")[2].partition("@")[2].partition("@")[0]
			descriptionin = line.partition("@")[2].partition("@")[2].partition("@")[2].strip("\n")
			loading.append([typein, quantityin, placein, descriptionin])
			line = filein.readline()
		#populating the new list
		newloading = []
		for i in range(len(loading)):
			found = 0
			for j in type:
				if loading[i][0] == j: found = 1
			if found == 0: newloading.append(loading[i])
		#writing out the new list
		fileout = open(listsdir+"/"+category+"s","w")
		j = 0
		while j<len(newloading):
			fileout.write(newloading[j][0]+"@"+str(newloading[j][1])+"@"+newloading[j][2]+"@"+newloading[j][3]+"\n")
			j = j+1
		fileout.close()
		loading = []
		newloading = []
	
	def search(self,category,depth,term):
		loading = []
		self.load_category(category)
		if term != "":
			for line in self.load:
				print depth[3]
				if depth[3] == True:
					if depth[0] == True:	#name search, case sensitive
						if line[0].partition(term)[1] != "": loading.append(line)
					if depth[1] == True:	#place search, case sensitive
						if line[2].partition(term)[1] != "": loading.append(line)
					if depth[2] == True:	#description search, case sensitive
						if line[3].partition(term)[1] != "": loading.append(line)
				else:
					if depth[0] == True:	#name search
						if line[0].upper().partition(term.upper())[1] != "": loading.append(line)
					if depth[1] == True:	#place search
						if line[2].upper().partition(term.upper())[1] != "": loading.append(line)
					if depth[2] == True:	#description search
						if line[3].upper().partition(term.upper())[1] != "": loading.append(line)
		return loading
	
	def empty_load(self):
		#Releases the contents of the loaded category by emptying the list.
		self.load = []
		self.category = ""
