#!/usr/bin/env python

import gtk, gtk.glade
import os, sys
#The sys.argv[0] is the relative path of the source file that's given to python.
#We change the working directory to the full path of that relative path.
os.chdir(os.path.abspath(os.path.dirname(sys.argv[0])))
cwdir = os.getcwd()
sys.path.append(cwdir+"/database")
import database
from random import randint

class GInterface:
	def __init__(self,catalogue):
		self.catalogue = catalogue
		self.selection = []
		self.selected_names = []
		self.selected_name = ""
		#loading in the glade file
		xml = gtk.glade.XML(cwdir+"/gui/ginterface.glade")
		#loading subwindows and connecting singnals
		self.addwindow = xml.get_widget("AddWindow")
		self.addwindow.connect("delete_event", self.addclose)
		self.removewindow = xml.get_widget("RemoveWindow")
		self.removewindow.connect("delete_event", self.removeclose)
		self.changewindow = xml.get_widget("ChangeWindow")
		self.changewindow.connect("delete_event", self.changeclose)
		self.removemorewindow = xml.get_widget("RemoveMoreWindow")
		self.removemorewindow.connect("delete_event", self.removeclose)
		self.aboutwindow = xml.get_widget("AboutWindow")
		self.aboutwindow.connect("delete_event", self.aboutclose)
		self.about_ok = xml.get_widget("CloseButton_about")
		self.about_ok.connect("clicked", self.aboutclose, 0)
		self.about_menubutton = xml.get_widget("AboutMenu")
		self.about_menubutton.connect("activate", self.about)
		self.clearwindow = xml.get_widget("ClearWindow")
		self.clearwindow.connect("delete_event", self.clearclose)
		self.clear_ok = xml.get_widget("YesButton_clear")
		self.clear_cancel = xml.get_widget("NoButton_clear")
		self.clear_ok.connect("clicked", self.clear)
		self.clear_cancel.connect("clicked", self.clearclose, 0)
		self.clear = xml.get_widget("ClearMenu")
		self.clear.connect("activate", self.clearing)
		self.quit = xml.get_widget("QuitMenu")
		self.quit.connect("activate", self.exit_loop)
		#warning subwindow
		self.warningwindow = xml.get_widget("WarningWindow")
		self.warningwindow.connect("delete_event", self.warningclose)
		self.warning_ok = xml.get_widget("OKButton_warning")
		self.warning_ok.connect("clicked", self.warningclose,0)
		self.warning_label = xml.get_widget("WarningLabel")
		#opening main window
		self.mainwindow = xml.get_widget("MainWindow")
		self.mainwindow.connect("destroy", self.exit_loop)
		#loading clist
		self.clist = xml.get_widget("CList")
		#loading category buttons and connecting signals
		self.cat_transistor = xml.get_widget("TransistorsButton")
		self.cat_transistor.connect("clicked", self.load_category, "transistor")
		self.cat_capacitor = xml.get_widget("CapacitorsButton")
		self.cat_capacitor.connect("clicked", self.load_category, "capacitor")
		self.cat_resistor = xml.get_widget("ResistorsButton")
		self.cat_resistor.connect("clicked", self.load_category, "resistor")
		self.cat_diode = xml.get_widget("DiodesButton")
		self.cat_diode.connect("clicked", self.load_category, "diode")
		self.cat_ic = xml.get_widget("IcsButton")
		self.cat_ic.connect("clicked", self.load_category, "ic")
		self.cat_button = xml.get_widget("ButtonsButton")
		self.cat_button.connect("clicked", self.load_category, "button")
		self.cat_other = xml.get_widget("OtherButton")
		self.cat_other.connect("clicked", self.load_category, "other")
		#loading sorting buttons and connecting signals
		self.sort_name = xml.get_widget("SortNameButton")
		self.sort_name.connect("clicked", self.sort, 0)
		self.sort_quantity = xml.get_widget("SortQuantityButton")
		self.sort_quantity.connect("clicked", self.sort, 1)
		self.sort_place = xml.get_widget("SortPlaceButton")
		self.sort_place.connect("clicked", self.sort, 2)
		self.sort_description = xml.get_widget("SortDescriptionButton")
		self.sort_description.connect("clicked", self.sort, 3)
		#loading the add button and connecting signals
		self.add_button = xml.get_widget("AddButton")
		self.add_menubutton = xml.get_widget("AddMenu")
		self.add_menubutton.connect("activate", self.adding)
		self.add_button.connect("clicked", self.adding)
		#loading the addwindow components
		self.add_nameentry = xml.get_widget("NameEntry_add")
		self.add_quantityentry = xml.get_widget("QuantityEntry_add")
		self.add_placeentry = xml.get_widget("PlaceEntry_add")
		self.add_descriptionentry = xml.get_widget("DescriptionEntry_add")
		self.add_ok = xml.get_widget("OKButton_add")
		self.add_ok.connect("clicked", self.add)
		self.add_cancel = xml.get_widget("CancelButton_add")
		self.add_cancel.connect("clicked", self.addclose, 0)
		#loading the remove button and connecting signals
		self.remove_button = xml.get_widget("RemoveButton")
		self.remove_menubutton = xml.get_widget("RemoveMenu")
		self.remove_menubutton.connect("activate", self.removing)
		self.remove_button.connect("clicked", self.removing)
		#loading the removewindow components
		self.remove_name = xml.get_widget("NameLabel_remove")
		self.remove_cancel = xml.get_widget("NoButton_remove")
		self.remove_cancel.connect("clicked", self.removeclose, 0)
		self.remove_ok = xml.get_widget("YesButton_remove")
		self.remove_ok.connect("clicked", self.remove)
		#removemorewindow components
		self.removemore_ok = xml.get_widget("YesButton_removemore")
		self.removemore_ok.connect("clicked", self.remove)
		self.removemore_cancel = xml.get_widget("NoButton_removemore")
		self.removemore_cancel.connect("clicked", self.removeclose, 0)
		#loading edit button and connecting signals
		self.edit_button = xml.get_widget("ChangeButton")
		self.edit_menubutton = xml.get_widget("ChangeMenu")
		self.edit_button.connect("clicked", self.changing)
		self.edit_menubutton.connect("activate", self.changing)
		#changewindow components
		self.change_nameentry = xml.get_widget("NameEntry_change")
		self.change_quantityentry = xml.get_widget("QuantityEntry_change")
		self.change_placeentry = xml.get_widget("PlaceEntry_change")
		self.change_descriptionentry = xml.get_widget("DescriptionEntry_change")
		self.change_ok = xml.get_widget("ChangeButton_change")
		self.change_ok.connect("clicked", self.change)
		self.change_cancel = xml.get_widget("CancelButton_change")
		self.change_cancel.connect("clicked", self.changeclose, 0)
		#loading the inrement/decrement buttons and connecting signals
		self.inc_button = xml.get_widget("IncButton")
		self.dec_button = xml.get_widget("DecButton")
		self.inc_button.connect("clicked", self.inc)
		self.dec_button.connect("clicked", self.dec)
		#loading the search components and connecting signals
		self.search_button = xml.get_widget("SearchButton")
		self.name_check = xml.get_widget("NameCheck")
		self.place_check = xml.get_widget("PlaceCheck")
		self.description_check = xml.get_widget("DescriptionCheck")
		self.case_check = xml.get_widget("CaseCheck")
		self.results_label = xml.get_widget("ResultsLabel")
		self.search_entry = xml.get_widget("SearchEntry")
		self.search_button.connect("clicked", self.search)
	
	def search(self,widget):
		self.clist.clear()
		if self.catalogue.category == "":
			self.warning_label.set_text("No category selected to search in.\nSelect a category by clicking one of the\ncategory buttons from the toolbar.")
			self.warningwindow.show()
		else:
			loading = catalogue.search(self.catalogue.category,[self.name_check.get_active(), self.place_check.get_active(), self.description_check.get_active(), self.case_check.get_active()],self.search_entry.get_text())
			for i in loading:
				self.clist.append([i[0],self.nullize(str(i[1])),i[2],i[3]])
			if len(loading) == 0: self.results_label.set_text("No Results in "+self.catalogue.category.title()+"s")
			elif len(loading) == 1: self.results_label.set_text("1 Result in "+self.catalogue.category.title()+"s")
			else: self.results_label.set_text(str(len(loading))+ " Results in "+self.catalogue.category.title()+"s")

	def inc(self,widget):
		if self.clist.selection != []:
			if int(self.clist.get_text(self.clist.selection[0],1))<100000:
				self.clist.set_text(self.clist.selection[0],1,self.nullize(str(int(self.clist.get_text(self.clist.selection[0],1))+1)))
				self.catalogue.set_attributes(self.catalogue.category,self.clist.get_text(self.clist.selection[0],0),quantity=self.clist.get_text(self.clist.selection[0],1))
				self.clist.sort()
		else:
			self.warning_label.set_text("Nothing selected!")
			self.warningwindow.show()

	def dec(self,widget):
		if self.clist.selection != []:
			if int(self.clist.get_text(self.clist.selection[0],1))>0:
				self.clist.set_text(self.clist.selection[0],1,self.nullize(str(int(self.clist.get_text(self.clist.selection[0],1))-1)))
				self.catalogue.set_attributes(self.catalogue.category,self.clist.get_text(self.clist.selection[0],0),quantity=self.clist.get_text(self.clist.selection[0],1))
				self.clist.sort()
		else:
			self.warning_label.set_text("Nothing selected!")
			self.warningwindow.show()			
	
	def removing(self,widget):
		if self.catalogue.category != "":
			if self.clist.selection != []:
				if len(self.clist.selection) == 1:
					self.selected_names.append(self.clist.get_text(self.clist.selection[0],0))
					self.remove_name.set_text(self.selected_names[0])
					self.removewindow.show()
				else:
					for i in self.clist.selection:
						self.selected_names.append(self.clist.get_text(i,0))
					self.removemorewindow.show()
			else:
				self.warning_label.set_text("Nothing selected for removal.\nSelect and element you wish to\nremove from the list.")
				self.warningwindow.show()
		else:
			self.warning_label.set_text("No category selected\nSelect a category by clicking one of the\ncategory buttons from the toolbar.")
			self.warningwindow.show()
				
	def remove(self,widget):
		self.catalogue.remove_element(self.catalogue.category, self.selected_names)
		self.load_category(0,self.catalogue.category)
		self.clist.sort()
		self.removeclose(widget,0)
	
	def removeclose(self,widget,data):
		self.removewindow.hide()
		self.removemorewindow.hide()
		self.selected_names = []
		self.selection = []
		return 1
	
	def adding(self,widget):
		if self.catalogue.category != "":
			self.addwindow.show()
		else:
			self.warning_label.set_text("No category selected\nSelect a category by clicking one of the\ncategory buttons from the toolbar.")
			self.warningwindow.show()
	
	def add(self,widget):
		if self.add_nameentry.get_text().partition("@")[1] == "" and self.add_quantityentry.get_text().partition("@")[1] == "" and self.add_placeentry.get_text().partition("@")[1] == "" and self.add_descriptionentry.get_text().partition("@")[1] == "":
			if self.add_nameentry.get_text() != "" and self.add_quantityentry.get_text() != "" and self.add_placeentry.get_text() != "" and self.add_descriptionentry.get_text() != "":
				try: int(self.add_quantityentry.get_text())
				except:
					self.warning_label.set_text('The "Quantity" entry has to be a number!')
					self.warningwindow.show()
				else:
					if int(self.add_quantityentry.get_text())>=0 and int(self.add_quantityentry.get_text())<100000:
						if self.catalogue.set_attributes(self.catalogue.category, self.add_nameentry.get_text()) == 0:
							self.catalogue.add_element(self.catalogue.category, self.add_nameentry.get_text(), self.add_quantityentry.get_text(), self.add_placeentry.get_text(), self.add_descriptionentry.get_text())
							self.clist.append([self.add_nameentry.get_text(), self.nullize(self.add_quantityentry.get_text()), self.add_placeentry.get_text(), self.add_descriptionentry.get_text()])
							self.clist.sort()
							self.addclose(widget,0)
						else:
							self.warning_label.set_text("Element already exists.")
							self.warningwindow.show()
					else:
						self.warning_label.set_text("The Quantity has to be a number between 0 and 99999!")
						self.warningwindow.show()
			else:
				self.warning_label.set_text("All the fields are mandatory!")
				self.warningwindow.show()
		else:
			self.warning_label.set_text('None of the fields may contain\nthe "@" character')
			self.warningwindow.show()
		
	
	def addclose(self,widget,data):
		self.add_nameentry.set_text("")
		self.add_quantityentry.set_text("")
		self.add_placeentry.set_text("")
		self.add_descriptionentry.set_text("")
		self.addwindow.hide()
		return 1
	
	def changing(self,widget):
		if self.catalogue.category != "":
			if self.clist.selection != []:
				self.selection = self.clist.selection[0]
				self.selected_name = self.clist.get_text(self.selection,0)
				self.change_nameentry.set_text(self.clist.get_text(self.clist.selection[0],0))
				self.change_quantityentry.set_text(self.clist.get_text(self.clist.selection[0],1))
				self.change_placeentry.set_text(self.clist.get_text(self.clist.selection[0],2))
				self.change_descriptionentry.set_text(self.clist.get_text(self.clist.selection[0],3))
				self.changewindow.show()
			else:
				self.warning_label.set_text("Nothing selected to edit.\nSelect and element you wish to\nedit from the list.")
				self.warningwindow.show()
		else:
			self.warning_label.set_text("No category selected\nSelect a category by clicking one of the\ncategory buttons from the toolbar.")
			self.warningwindow.show()
	
	def change(self,widget):
		if self.change_nameentry.get_text().partition("@")[1] == "" and self.change_quantityentry.get_text().partition("@")[1] == "" and self.change_placeentry.get_text().partition("@")[1] == "" and self.change_descriptionentry.get_text().partition("@")[1] == "":
			if self.change_nameentry.get_text() != "" and self.change_quantityentry.get_text() != "" and self.change_placeentry.get_text() != "" and self.change_descriptionentry.get_text() != "":
				try: int(self.change_quantityentry.get_text())
				except:
					self.warning_label.set_text('The "Quantity" entry has to be a number!')
					self.warningwindow.show()
				else:
					if int(self.change_quantityentry.get_text())>=0 and int(self.change_quantityentry.get_text())<100000:
						self.catalogue.set_attributes(self.catalogue.category, self.selected_name, self.change_quantityentry.get_text(), self.change_placeentry.get_text(), self.change_descriptionentry.get_text(), self.change_nameentry.get_text())
						self.clist.set_text(self.selection,0,self.change_nameentry.get_text())
						self.clist.set_text(self.selection,1,self.nullize(self.change_quantityentry.get_text()))
						self.clist.set_text(self.selection,2,self.change_placeentry.get_text())
						self.clist.set_text(self.selection,3,self.change_descriptionentry.get_text())
						self.clist.sort()
						self.changeclose(widget,0)
					else:
						self.warning_label.set_text("The Quantity has to be a number between 0 and 99999!")
						self.warningwindow.show()
			else:
				self.warning_label.set_text("All the fields are mandatory!")
				self.warningwindow.show()
		else:
			self.warning_label.set_text('None of the fields may contain\nthe "@" character')
			self.warningwindow.show()		
	
	def changeclose(self,widget,data):
		self.changewindow.hide()
		return 1
	
	def clearing(self,widget):
		self.clearwindow.show()
	
	def clear(self,widget):
		self.catalogue.reset_all()
		self.clist.clear()
		self.clearclose(widget,0)
	
	def clearclose(self,widget,data):
		self.clearwindow.hide()
		return 1
	
	def warningclose(self,widget,data):
		self.warningwindow.hide()
		return 1
	
	def about(self,widget):
		self.aboutwindow.show()
	
	def aboutclose(self,widget,data):
		self.aboutwindow.hide()
		return 1
	
	def sort(self, widget, column):
		self.clist.set_sort_column(column)
		self.clist.sort()
	
	def nullize_all(self):
		largest = 5
		for i in range(len(catalogue.load)):
			if len(str(catalogue.load[i][1]))<largest: catalogue.load[i][1] = str("0"*(largest-len(str(catalogue.load[i][1])))+str(catalogue.load[i][1]))

	def nullize(self,string):
		largest = 5
		if len(string)<largest: string = "0"*(largest-len(string))+string
		return string
	
	def load_category(self, widget, category):
		self.clist.clear()
		self.catalogue.load_category(category)
		self.nullize_all()
		for i in range(len(self.catalogue.load)):
			self.clist.append([self.catalogue.load[i][0],str(self.catalogue.load[i][1]),self.catalogue.load[i][2],self.catalogue.load[i][3]])
		self.results_label.set_text(self.catalogue.category.title()+"s")
	
	def exit_loop(self, widget, data=None):
		gtk.main_quit()

	def enter_loop(self):
		gtk.main()

#Loading Catalogue
catalogue = database.ElectroCatalogue(cwdir+"/database/lists",cwdir)

#Starting GUI
ginterface = GInterface(catalogue)
ginterface.enter_loop()
